package ink.wgink.code.factory.controller;

import freemarker.template.TemplateException;
import ink.wgink.code.factory.factory.ButtonTableCellFactory;
import ink.wgink.code.factory.factory.CheckBoxTableCellFactory;
import ink.wgink.code.factory.factory.ChoiceBoxTableCellFactory;
import ink.wgink.code.factory.service.FieldService;
import ink.wgink.code.factory.service.GenerateService;
import ink.wgink.code.factory.utils.MsgUtil;
import ink.wgink.code.factory.vos.FieldVO;
import ink.wgink.code.factory.vos.GenerateVO;
import ink.wgink.code.factory.vos.TableVO;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.stage.DirectoryChooser;
import javafx.stage.Stage;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ResourceBundle;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: GenerateController
 * @Description: 生成器
 * @Author: WangGeng
 * @Date: 2021/3/6 10:11
 * @Version: 1.0
 **/
public class GenerateController implements Initializable {

    @FXML
    private CheckBox apiCheckBox;
    @FXML
    private CheckBox appCheckBox;
    @FXML
    private CheckBox resourceCheckBox;
    @FXML
    private CheckBox iServiceCheckBox;
    @FXML
    private CheckBox serviceImplCheckBox;
    @FXML
    private CheckBox iDaoCheckBox;
    @FXML
    private CheckBox mysqlCheckBox;
    @FXML
    private TextField tablePrefixTextField;
    @FXML
    private CheckBox htmlCheckBox;
    @FXML
    private CheckBox thymeleafCheckBox;
    @FXML
    private CheckBox routeCheckBox;
    @FXML
    private TextField contextTextField;
    @FXML
    private TextField packageTextField;
    @FXML
    private TableView fieldTableView;

    private TableVO tableVO;
    private FieldService fieldService;
    private GenerateService generateService;
    private Stage fieldStage;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        fieldService = new FieldService();
        initTableView();
    }

    private void initTableView() {
        ObservableList<TableColumn> tableColumns = fieldTableView.getColumns();
        tableColumns.get(0).setCellValueFactory(new PropertyValueFactory<>("rowNumber"));
        tableColumns.get(1).setCellValueFactory(new PropertyValueFactory<>("columnName"));
        tableColumns.get(2).setCellValueFactory(new PropertyValueFactory<>("columnComment"));
        tableColumns.get(3).setCellValueFactory(new PropertyValueFactory<>("columnDefault"));
        tableColumns.get(4).setCellValueFactory(new PropertyValueFactory<>("columnType"));
        tableColumns.get(5).setCellValueFactory(new PropertyValueFactory<>("propertyName"));
        tableColumns.get(6).setCellValueFactory(new PropertyValueFactory<>("propertyType"));
        tableColumns.get(7).setCellValueFactory(new PropertyValueFactory<>("propertyLength"));
        tableColumns.get(8).setCellValueFactory(new CheckBoxTableCellFactory("isFormShow"));
        tableColumns.get(9).setCellValueFactory(new CheckBoxTableCellFactory("isListShow"));
        tableColumns.get(10).setCellValueFactory(new CheckBoxTableCellFactory("isNotNull"));
        tableColumns.get(11).setCellValueFactory(new ChoiceBoxTableCellFactory("formFieldType"));
        tableColumns.get(12).setCellValueFactory(new ButtonTableCellFactory("option", fieldService));
    }

    public void setTableVO(TableVO tableVO) throws IOException, URISyntaxException {
        this.tablePrefixTextField.setText(tableVO.getTableName().split("\\_")[0] + "_");
        this.tableVO = tableVO;
        this.generateService = new GenerateService(fieldTableView);
        this.fieldService.setTableView(fieldTableView);
        this.fieldService.setTableName(tableVO.getTableName());
        this.fieldService.showField();
    }

    @FXML
    public void onGenerateClick(ActionEvent actionEvent) throws IOException, TemplateException {
        ObservableList<FieldVO> items = fieldTableView.getItems();
        for (FieldVO fieldVO : items) {
            if (StringUtils.isBlank(fieldVO.getFormFieldValue())) {
                MsgUtil.errorAlert(fieldVO.getColumnName() + "的表单类型不能为空");
                return;
            }
        }
        GenerateVO generateVO = new GenerateVO();
        generateVO.setHasApi(apiCheckBox.isSelected());
        generateVO.setHasApp(appCheckBox.isSelected());
        generateVO.setHasResource(resourceCheckBox.isSelected());
        generateVO.setHasIService(iServiceCheckBox.isSelected());
        generateVO.setHasServiceImpl(serviceImplCheckBox.isSelected());
        generateVO.setHasIDao(iDaoCheckBox.isSelected());
        generateVO.setHasMySQL(mysqlCheckBox.isSelected());
        generateVO.setHasHtml(htmlCheckBox.isSelected());
        generateVO.setHasThymeleaf(thymeleafCheckBox.isSelected());
        generateVO.setHasRoute(routeCheckBox.isSelected());
        generateVO.setTablePrefix(tablePrefixTextField.getText());
        generateVO.setContext(contextTextField.getText());
        generateVO.setCodePackage(packageTextField.getText());
        generateService.generateCode(tableVO, generateVO);
    }

    @FXML
    public void onRefreshClick(ActionEvent actionEvent) throws IOException {
        this.fieldService.showField();
    }

    public void setFieldStage(Stage fieldStage) {
        this.fieldStage = fieldStage;
        fieldService.setFieldStage(fieldStage);
        generateService.setFieldStage(fieldStage);
    }
}
