package ink.wgink.code.factory.controller;

import ink.wgink.code.factory.service.TableService;
import ink.wgink.code.factory.utils.MsgUtil;
import ink.wgink.code.factory.vos.TableVO;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ResourceBundle;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: MainController
 * @Description: 主页
 * @Author: WangGeng
 * @Date: 2021/3/3 20:02
 * @Version: 1.0
 **/
public class MainController implements Initializable {

    @FXML
    private TextField dbUrlField;
    @FXML
    private TextField dbPortField;
    @FXML
    private TextField dbNameField;
    @FXML
    private TextField dbUsernameField;
    @FXML
    private PasswordField dbPasswordField;
    @FXML
    private Button connectionDatabaseButton;
    @FXML
    private TableView dbTableTableView;
    private TableService tableService;
    private Stage mainStage;


    @Override
    public void initialize(URL location, ResourceBundle resources) {
        initDBTableTableView();
        this.tableService = new TableService(dbTableTableView);
    }

    /**
     * 初始化 数据库表 TableView
     */
    private void initDBTableTableView() {
        ObservableList<TableColumn<TableVO, String>> tableColumns = dbTableTableView.getColumns();
        tableColumns.get(0).setCellValueFactory(new PropertyValueFactory<>("rowNumber"));
        tableColumns.get(1).setCellValueFactory(new PropertyValueFactory<>("tableName"));
        tableColumns.get(2).setCellValueFactory(new PropertyValueFactory<>("tableComment"));
        tableColumns.get(3).setCellValueFactory(new PropertyValueFactory<>("tableCollation"));
    }

    @FXML
    public void onConnectDatabase(ActionEvent event) throws IOException {
        String dbUrl = dbUrlField.getText();
        if (StringUtils.isBlank(dbUrl)) {
            MsgUtil.errorAlert("数据库地址不能为空");
            return;
        }
        String dbPort = dbPortField.getText();
        if (StringUtils.isBlank(dbPort)) {
            MsgUtil.errorAlert("数据库端口不能为空");
            return;
        }
        String dbName = dbNameField.getText();
        if (StringUtils.isBlank(dbName)) {
            MsgUtil.errorAlert("数据库名不能为空");
            return;
        }
        String dbUsername = dbUsernameField.getText();
        if (StringUtils.isBlank(dbUsername)) {
            MsgUtil.errorAlert("数据库用户名不能为空");
            return;
        }
        String dbPassword = dbPasswordField.getText();
        if (StringUtils.isBlank(dbPassword)) {
            MsgUtil.errorAlert("数据库密码不能为空");
            return;
        }

        tableService.connectionDatabase(dbUrl, dbPort, dbName, dbUsername, dbPassword);
    }

    @FXML
    public void onTableRowClick(MouseEvent event) throws IOException, URISyntaxException {
        if (event.getClickCount() == 2) {
            TableVO tableVO = (TableVO) this.dbTableTableView.getSelectionModel().getSelectedItem();
            FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("/route/generate.fxml"));
            Stage stage = new Stage(StageStyle.DECORATED);
            stage.setScene(new Scene(fxmlLoader.load(), 800D, 600D));
            stage.initModality(Modality.APPLICATION_MODAL);
            stage.setTitle("生成代码【" + tableVO.getTableName() + "】");
            stage.show();

            GenerateController generateController = fxmlLoader.getController();
            generateController.setTableVO(tableVO);
        }
    }

    @FXML
    public void dbUrlReleased(KeyEvent keyEvent) {
        String port = dbUrlField.getText();
        if (StringUtils.isBlank(port)) {
            dbUrlField.setText("");
            return;
        }
    }

    @FXML
    public void dbNameReleased(KeyEvent keyEvent) {
        String port = dbNameField.getText();
        if (StringUtils.isBlank(port)) {
            dbNameField.setText("");
            return;
        }
    }

    @FXML
    public void dbUsernameReleased(KeyEvent keyEvent) {
        String port = dbUsernameField.getText();
        if (StringUtils.isBlank(port)) {
            dbUsernameField.setText("");
            return;
        }
    }

    @FXML
    public void dbPortReleased(KeyEvent keyEvent) {
        String port = dbPortField.getText();
        if (StringUtils.isBlank(port)) {
            dbPortField.setText("");
            return;
        }
        if (!NumberUtils.isDigits(port)) {
            MsgUtil.alert(AlertType.ERROR, "端口号只能是数字");
            dbPortField.setText("");
        }
    }

    public void setMainStage(Stage mainStage) {
        this.mainStage = mainStage;
        this.tableService.setMainStage(mainStage);
    }
}
