package ink.wgink.code.factory.manager;

import ink.wgink.code.factory.utils.JdbcUtil;

import java.sql.*;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: JdbcManager
 * @Description: JDBC管理
 * @Author: WangGeng
 * @Date: 2021/3/5 21:23
 * @Version: 1.0
 **/
public class JdbcManager {

    private static JdbcManager jdbcManager = JdbcManagerBuilder.jdbcManager;
    private String dbUrl;
    private String dbPort;
    private String dbName;
    private String dbUsername;
    private String dbPassword;
    private Connection connection;

    static {
        try {
            Class.forName("com.mysql.jdbc.Driver");
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    private JdbcManager() {
    }

    public static JdbcManager getInstance() {
        return jdbcManager;
    }

    public void setDatabaseInfo(String dbUrl, String dbPort, String dbName, String dbUsername, String dbPassword) {
        this.dbUrl = dbUrl;
        this.dbPort = dbPort;
        this.dbName = dbName;
        this.dbUsername = dbUsername;
        this.dbPassword = dbPassword;
    }

    /**
     * 数据库链接
     *
     * @return
     * @throws SQLException
     */
    public Connection getConnection() throws SQLException {
        if (connection == null || connection.isClosed()) {
            this.connection = DriverManager.getConnection(String.format("jdbc:mysql://%s:%s/%s", dbUrl, dbPort, dbName), dbUsername, dbPassword);
        }
        return connection;
    }

    /**
     * 数据库表列表
     *
     * @return
     * @throws SQLException
     */
    public List<Map<String, Object>> listDatabaseTable() throws SQLException {
        String SQL = "SELECT * FROM `information_schema`.`TABLES` WHERE TABLE_SCHEMA = ?";
        PreparedStatement preparedStatement = getConnection().prepareStatement(SQL);
        preparedStatement.setString(1, this.dbName);
        return JdbcUtil.listResult(preparedStatement.executeQuery());
    }

    /**
     * 表字段列表
     *
     * @param tableName
     * @return
     * @throws SQLException
     */
    public List<Map<String, Object>> listTableField(String tableName) throws SQLException {
        String SQL = "SELECT * FROM `information_schema`.`COLUMNS` WHERE TABLE_SCHEMA = ? AND TABLE_NAME = ?";
        PreparedStatement preparedStatement = getConnection().prepareStatement(SQL);
        preparedStatement.setString(1, this.dbName);
        preparedStatement.setString(2, tableName);
        return JdbcUtil.listResult(preparedStatement.executeQuery());
    }

    private static class JdbcManagerBuilder {
        public static JdbcManager jdbcManager = new JdbcManager();
    }

}
