package ink.wgink.code.factory.service;

import ink.wgink.code.factory.enums.ColumnDataTypeEnum;
import ink.wgink.code.factory.enums.ColumnIsNullableEnum;
import ink.wgink.code.factory.enums.FormFieldTypeEnum;
import ink.wgink.code.factory.enums.PropertyTypeEnum;
import ink.wgink.code.factory.manager.JdbcManager;
import ink.wgink.code.factory.utils.MsgUtil;
import ink.wgink.code.factory.utils.WStringUtil;
import ink.wgink.code.factory.vos.FieldVO;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.scene.control.TableView;
import javafx.stage.Stage;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: FieldService
 * @Description: 字段业务
 * @Author: WangGeng
 * @Date: 2021/3/6 16:11
 * @Version: 1.0
 **/
public class FieldService {

    public static final String DEFAULT_FIELD_ID = "id";
    public static final String DEFAULT_FIELD_CREATOR = "creator";
    public static final String DEFAULT_COLUMN_CREATOR = "creator";
    public static final String DEFAULT_FIELD_GMT_CREATE = "gmtCreate";
    public static final String DEFAULT_COLUMN_GMT_CREATE = "gmt_create";
    public static final String DEFAULT_FIELD_MODIFIER = "modifier";
    public static final String DEFAULT_COLUMN_MODIFIER = "modifier";
    public static final String DEFAULT_FIELD_GMT_MODIFIED = "gmtModified";
    public static final String DEFAULT_COLUMN_GMT_MODIFIED = "gmt_modified";
    public static final String DEFAULT_FIELD_IS_DELETE = "isDelete";
    public static final String DEFAULT_COLUMN_IS_DELETE = "is_delete";

    private ObservableList<FieldVO> fields = FXCollections.observableArrayList();
    private TableView tableView;
    private String tableName;
    private Stage fieldStage;

    /**
     * 表单字段类型
     *
     * @param columnDataType
     * @return
     */
    public String getFormFieldType(String columnDataType) {
        if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.INT.getValue(), columnDataType)) {
            return FormFieldTypeEnum.NUMBER.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.BIGINT.getValue(), columnDataType)) {
            return FormFieldTypeEnum.NUMBER.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.DOUBLE.getValue(), columnDataType)) {
            return FormFieldTypeEnum.DOUBLE.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.DATETIME.getValue(), columnDataType)) {
            return FormFieldTypeEnum.DATETIME.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.DATE.getValue(), columnDataType)) {
            return FormFieldTypeEnum.DATE.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.TEXT.getValue(), columnDataType)) {
            return FormFieldTypeEnum.TEXTAREA.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.LONGTEXT.getValue(), columnDataType)) {
            return FormFieldTypeEnum.RICH_TEXT.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.CHAR.getValue(), columnDataType)) {
            return FormFieldTypeEnum.ID_SELECT.getValue();
        }
        return FormFieldTypeEnum.TEXT.getValue();
    }

    /**
     * Java属性栏类型
     *
     * @param columnDataType
     * @return
     */
    public String getPropertyType(String columnDataType) {
        if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.INT.getValue(), columnDataType)) {
            return PropertyTypeEnum.INTEGER.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.BIGINT.getValue(), columnDataType)) {
            return PropertyTypeEnum.LONG.getValue();
        } else if (StringUtils.equalsIgnoreCase(ColumnDataTypeEnum.DOUBLE.getValue(), columnDataType)) {
            return PropertyTypeEnum.DOUBLE.getValue();
        }
        return PropertyTypeEnum.STRING.getValue();
    }

    public void showField() throws IOException {
        MsgUtil.loading(fieldStage);
        new Service<Integer>() {

            @Override
            protected Task<Integer> createTask() {
                return new Task<Integer>() {
                    @Override
                    protected Integer call() throws Exception {
                        List<Map<String, Object>> tableFields = JdbcManager.getInstance().listTableField(tableName);
                        fields.clear();
                        int fieldIndex = 1;
                        for (Map<String, Object> field : tableFields) {
                            String columnName = field.get("COLUMN_NAME").toString();
                            if (StringUtils.equals(DEFAULT_FIELD_ID, columnName)) {
                                continue;
                            }
                            if (StringUtils.equals(DEFAULT_FIELD_IS_DELETE, columnName)) {
                                continue;
                            }
                            FieldVO fieldVO = new FieldVO();
                            fieldVO.setRowNumber(fieldIndex);
                            fieldVO.setColumnName(columnName);
                            fieldVO.setColumnType(field.get("COLUMN_TYPE").toString());
                            fieldVO.setDataType(field.get("DATA_TYPE").toString());
                            fieldVO.setCharacterMaximum(field.get("CHARACTER_MAXIMUM_LENGTH") == null ? 0 : Integer.valueOf(field.get("CHARACTER_MAXIMUM_LENGTH").toString()));
                            fieldVO.setNumericPrecision(field.get("NUMERIC_PRECISION") == null ? 0 : Integer.valueOf(field.get("NUMERIC_PRECISION").toString()));
                            fieldVO.setNumericScale(field.get("NUMERIC_SCALE") == null ? 0 : Integer.valueOf(field.get("NUMERIC_SCALE").toString()));
                            fieldVO.setColumnComment(field.get("COLUMN_COMMENT") == null ? "" : field.get("COLUMN_COMMENT").toString());
                            fieldVO.setColumnDefault(field.get("COLUMN_DEFAULT") == null ? "" : field.get("COLUMN_DEFAULT").toString());
                            fieldVO.setPropertyName(WStringUtil.underLine2LowerUpper(fieldVO.getColumnName()));
                            if (fieldVO.getNumericPrecision() != 0) {
                                fieldVO.setPropertyLength(fieldVO.getNumericPrecision());
                            } else {
                                fieldVO.setPropertyLength(fieldVO.getCharacterMaximum());
                            }
                            fieldVO.setNotNull(StringUtils.equalsIgnoreCase(ColumnIsNullableEnum.YES.getValue(), field.get("IS_NULLABLE").toString()) ? false : true);
                            fieldVO.setPropertyType(getPropertyType(fieldVO.getDataType()));
                            fieldVO.setFormFieldType(getFormFieldType(fieldVO.getDataType()));
                            fields.add(fieldVO);
                            fieldIndex++;
                        }
                        Thread.sleep(1000);
                        return 0;
                    }
                };
            }

            @Override
            protected void succeeded() {
                tableView.setItems(fields);
                MsgUtil.closeNewLoading();
                super.succeeded();
            }
        }.start();
    }

    public ObservableList<FieldVO> getFields() {
        return fields;
    }

    public TableView getTableView() {
        return tableView;
    }

    public void setTableView(TableView tableView) {
        this.tableView = tableView;
    }

    public String getTableName() {
        return tableName == null ? "" : tableName;
    }

    public void setTableName(String tableName) {
        this.tableName = tableName;
    }

    public void removeField(String columnName) {
        new Service<Integer>() {

            @Override
            protected Task<Integer> createTask() {
                return new Task<Integer>() {
                    @Override
                    protected Integer call() throws Exception {
                        for (int i = 0; i < fields.size(); i++) {
                            FieldVO fieldVO = fields.get(i);
                            if (StringUtils.equals(fieldVO.getColumnName(), columnName)) {
                                fields.remove(i);
                                i--;
                                continue;
                            }
                            fieldVO.setRowNumber(i + 1);
                        }
                        return 0;
                    }

                    @Override
                    protected void succeeded() {
                        tableView.setItems(fields);
                        super.succeeded();
                    }
                };
            }

        }.start();
    }

    public void setFieldStage(Stage fieldStage) {
        this.fieldStage = fieldStage;
    }
}
