package ink.wgink.code.factory.service;

import freemarker.cache.ClassTemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateException;
import ink.wgink.code.factory.utils.DateUtil;
import ink.wgink.code.factory.utils.MsgUtil;
import ink.wgink.code.factory.utils.WStringUtil;
import ink.wgink.code.factory.vos.FieldVO;
import ink.wgink.code.factory.vos.GenerateVO;
import ink.wgink.code.factory.vos.TableVO;
import javafx.collections.ObservableList;
import javafx.scene.control.Alert;
import javafx.scene.control.TableView;
import javafx.stage.DirectoryChooser;
import javafx.stage.Stage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: GenerateService
 * @Description: 代码生成
 * @Author: WangGeng
 * @Date: 2021/3/6 16:11
 * @Version: 1.0
 **/
@Slf4j
public class GenerateService {

    private Stage fieldStage;
    private TableView tableView;
    Configuration freemarkerConfiguration = new Configuration(Configuration.getVersion());

    public GenerateService(TableView tableView) throws URISyntaxException, IOException {
        this.tableView = tableView;
        // 1.加载 文件夹里的模板
        // this.freemarkerConfiguration.setDirectoryForTemplateLoading(new File(getClass().getResource("/template").toURI()));
        // 2.加载 jar中的模板
        // this.freemarkerConfiguration.setTemplateLoader(new ClassTemplateLoader(this.getClass(), "/template"));
        // 3.加载 jar中的模板
        this.freemarkerConfiguration.setClassForTemplateLoading(this.getClass(), "/template");
        this.freemarkerConfiguration.setDefaultEncoding("utf-8");
    }

    public void generateCode(TableVO tableVO, GenerateVO generateVO) throws IOException, TemplateException {
        // 选择文件夹
        DirectoryChooser directoryChooser = new DirectoryChooser();
        File outFolderFile = directoryChooser.showDialog(fieldStage);
        if (outFolderFile == null) {
            return;
        }
        String outFolder = outFolderFile.getPath();
        // 无前缀表名
        String tableFullName = tableVO.getTableName();
        String tableNameWithoutPrefix = tableFullName.replaceFirst(generateVO.getTablePrefix(), "");
        String tableName = WStringUtil.underLine2LowerUpper(tableNameWithoutPrefix);
        // 首字母大写驼峰表名
        String firstUpperTableName = WStringUtil.firstToUpper(tableName);
        // 首字母小写驼峰表明
        String firstLowerTableName = WStringUtil.firstToLower(tableName);
        String lowerTableName = tableNameWithoutPrefix.replaceAll("\\_", "").toLowerCase();
        Map<String, Object> dataModel = new HashMap<>(16);
        dataModel.put("context", generateVO.getContext());
        dataModel.put("codePackage", generateVO.getCodePackage());
        dataModel.put("tableNameWithoutPrefix", tableNameWithoutPrefix);
        dataModel.put("lowerTableName", lowerTableName);
        dataModel.put("tableName", tableName);
        dataModel.put("tableFullName", tableFullName);
        dataModel.put("firstUpperTableName", firstUpperTableName);
        dataModel.put("firstLowerTableName", firstLowerTableName);
        dataModel.put("tableExplain", tableVO.getTableComment());
        dataModel.put("author", "CodeFactory");
        dataModel.put("date", DateUtil.getTime());
        dataModel.put("version", "3.0");
        initFieldList(tableNameWithoutPrefix, dataModel, tableView.getItems());
        // 生成API
        if (generateVO.getHasApi()) {
            apiCode("/normal/controller/api-controller.ftl", String.format("%s/controller/api/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        }
        if (generateVO.getHasApi()) {
            appCode("/normal/controller/app-controller.ftl", String.format("%s/controller/app/api/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        }
        if (generateVO.getHasResource()) {
            resourceCode("/normal/controller/resource-controller.ftl", String.format("%s/controller/resource/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        }
        if (generateVO.getHasIService()) {
            iServiceCode("/normal/service/i-service.ftl", String.format("%s/service/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        }
        if (generateVO.getHasServiceImpl()) {
            serviceImplCode("/normal/service/service-impl.ftl", String.format("%s/service/%s/impl", outFolder, lowerTableName), firstUpperTableName, dataModel);
        }
        if (generateVO.getHasIDao()) {
            iDaoCode("/normal/dao/i-dao.ftl", String.format("%s/dao/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        }
        if (generateVO.getHasMySQL()) {
            mySqlMapperCode("/normal/mapper/mysql-mapper.ftl", String.format("%s/mapper/%s", outFolder, tableName), tableName, dataModel);
        }
        if (generateVO.getHasHtml()) {
            htmlCode("/normal/page", String.format("%s/static/route/%s", outFolder, tableName), tableName, dataModel);
        }
        if (generateVO.getHasThymeleaf()) {
            thymeleafCode("/normal/page", String.format("%s/templates/%s", outFolder, tableName), tableName, dataModel);
        }
        if (generateVO.getHasRoute()) {
            routeCode("/normal/controller/route-controller.ftl", String.format("%s/controller/route/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        }
        boCode("/normal/pojo/bo.ftl", String.format("%s/pojo/bos/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        dtoCode("/normal/pojo/dto.ftl", String.format("%s/pojo/dtos/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        poCode("/normal/pojo/po.ftl", String.format("%s/pojo/pos/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        voCode("/normal/pojo/vo.ftl", String.format("%s/pojo/vos/%s", outFolder, lowerTableName), firstUpperTableName, dataModel);
        MsgUtil.alert(Alert.AlertType.CONFIRMATION, "生成成功");
    }

    /**
     * 初始化字段
     *
     * @param dataModel
     */
    private void initFieldList(String tableNameWithoutPrefix, Map<String, Object> dataModel, ObservableList<FieldVO> fields) {
        List<Map<String, Object>> fieldList = new ArrayList<>();
        boolean hasId = false;
        boolean hasGmtCreate = false;
        boolean hasCreator = false;
        boolean hasGmtModified = false;
        boolean hasModifier = false;
        boolean hasIsDelete = false;
        for (int i = 0; i < fields.size(); i++) {
            FieldVO field = fields.get(i);
            if (!hasId && StringUtils.equals(String.format("%s_id", tableNameWithoutPrefix), field.getColumnName())) {
                hasId = true;
            }
            if (!hasGmtCreate && StringUtils.equals(FieldService.DEFAULT_FIELD_GMT_CREATE, field.getPropertyName())) {
                hasGmtCreate = true;
            }
            if (!hasCreator && StringUtils.equals(FieldService.DEFAULT_FIELD_CREATOR, field.getPropertyName())) {
                hasCreator = true;
            }
            if (!hasGmtModified && StringUtils.equals(FieldService.DEFAULT_FIELD_GMT_MODIFIED, field.getPropertyName())) {
                hasGmtModified = true;
            }
            if (!hasModifier && StringUtils.equals(FieldService.DEFAULT_FIELD_MODIFIER, field.getPropertyName())) {
                hasModifier = true;
            }
            if (!hasIsDelete && StringUtils.equals(FieldService.DEFAULT_FIELD_IS_DELETE, field.getPropertyName())) {
                hasIsDelete = true;
            }
            Map<String, Object> fieldMap = new HashMap<>(16);
            fieldMap.put("columnName", field.getColumnName());
            fieldMap.put("propertyName", field.getPropertyName());
            fieldMap.put("firstUpperPropertyName", WStringUtil.firstToUpper(field.getPropertyName()));
            fieldMap.put("propertyType", field.getPropertyType());
            fieldMap.put("columnComment", field.getColumnComment());
            fieldMap.put("formFieldValue", field.getFormFieldValue());
            fieldMap.put("columnDefault", field.getColumnDefault());
            fieldMap.put("propertyLength", field.getPropertyLength());
            fieldMap.put("formShow", field.getFormShow());
            fieldMap.put("listShow", field.getListShow());
            String fieldSplit = "";
            if (i < fields.size() - 1) {
                fieldSplit = ",";
            }
            fieldMap.put("fieldSplit", fieldSplit);
            fieldList.add(fieldMap);
        }
        dataModel.put("hasId", hasId);
        dataModel.put("hasGmtCreate", hasGmtCreate);
        dataModel.put("hasCreator", hasCreator);
        dataModel.put("hasGmtModified", hasGmtModified);
        dataModel.put("hasModifier", hasModifier);
        dataModel.put("hasIsDelete", hasIsDelete);
        dataModel.put("fieldList", fieldList);
    }

    /**
     * api-controller
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void apiCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sController.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * app-controller
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void appCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sAppController.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * resource-controller
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void resourceCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sResourceController.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * i-service
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void iServiceCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/I%sService.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * service-impl
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void serviceImplCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sServiceImpl.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * i-dao-impl
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void iDaoCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/I%sDao.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * mysql-mapper
     *
     * @param templateFtl
     * @param outFolder
     * @param tableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void mySqlMapperCode(String templateFtl, String outFolder, String tableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%s-mapper.xml", outFolder, tableName), dataModel);
    }

    /**
     * bo
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void boCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sBO.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * dto
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void dtoCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sDTO.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * po
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void poCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sPO.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * po
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void voCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sVO.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * html
     *
     * @param templateFolder
     * @param outFolder
     * @param tableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void htmlCode(String templateFolder, String outFolder, String tableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        dataModel.put("html", true);
        dataModel.put("thymeleaf", false);
        code(String.format("%s/list.ftl", templateFolder), String.format("%s/list.html", outFolder), dataModel);
        code(String.format("%s/save.ftl", templateFolder), String.format("%s/save.html", outFolder), dataModel);
        code(String.format("%s/update.ftl", templateFolder), String.format("%s/update.html", outFolder), dataModel);
    }

    /**
     * thymeleaf
     *
     * @param templateFolder
     * @param outFolder
     * @param tableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void thymeleafCode(String templateFolder, String outFolder, String tableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        dataModel.put("html", false);
        dataModel.put("thymeleaf", true);
        code(String.format("%s/list.ftl", templateFolder), String.format("%s/list.html", outFolder), dataModel);
        code(String.format("%s/save.ftl", templateFolder), String.format("%s/save.html", outFolder), dataModel);
        code(String.format("%s/update.ftl", templateFolder), String.format("%s/update.html", outFolder), dataModel);
    }

    /**
     * route-controller
     *
     * @param templateFtl
     * @param outFolder
     * @param firstUpperTableName
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void routeCode(String templateFtl, String outFolder, String firstUpperTableName, Map<String, Object> dataModel) throws IOException, TemplateException {
        File folder = new File(outFolder);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        code(templateFtl, String.format("%s/%sRouteController.java", outFolder, firstUpperTableName), dataModel);
    }

    /**
     * 生成代码
     *
     * @param templateFtl
     * @param outFile
     * @param dataModel
     * @throws IOException
     * @throws TemplateException
     */
    private void code(String templateFtl, String outFile, Map<String, Object> dataModel) throws IOException, TemplateException {
        Template template = freemarkerConfiguration.getTemplate(templateFtl);
        Writer out = new FileWriter(outFile);
        template.process(dataModel, out);
        out.close();
    }

    public void setFieldStage(Stage fieldStage) {
        this.fieldStage = fieldStage;
    }
}
