package ink.wgink.code.factory.service;

import ink.wgink.code.factory.manager.JdbcManager;
import ink.wgink.code.factory.utils.MsgUtil;
import ink.wgink.code.factory.vos.TableVO;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.scene.control.TableView;
import javafx.stage.Stage;

import java.io.IOException;
import java.util.List;
import java.util.Map;


/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: TableService
 * @Description: 表
 * @Author: WangGeng
 * @Date: 2021/3/5 21:18
 * @Version: 1.0
 **/
public class TableService {

    private Stage mainStage;
    private TableView dbTableTableView;
    private ObservableList<TableVO> tableVOObservableList = FXCollections.observableArrayList();

    public TableService(TableView dbTableTableView) {
        this.dbTableTableView = dbTableTableView;
    }

    /**
     * 链接数据库
     *
     * @param dbUrl      数据库链接
     * @param dbPort     数据库端口
     * @param dbName     数据库名称
     * @param dbUsername 数据库用户名
     * @param dbPassword 数据库密码
     */
    public void connectionDatabase(String dbUrl, String dbPort, String dbName, String dbUsername, String dbPassword) throws IOException {
        JdbcManager.getInstance().setDatabaseInfo(dbUrl, dbPort, dbName, dbUsername, dbPassword);
        showTable();
    }

    private void showTable() throws IOException {
        MsgUtil.loading(mainStage);
        new Service<Integer>() {
            @Override
            protected Task<Integer> createTask() {
                return new Task<Integer>() {
                    @Override
                    protected Integer call() throws Exception {
                        List<Map<String, Object>> tables = JdbcManager.getInstance().listDatabaseTable();
                        tableVOObservableList.clear();
                        for (int i = 0; i < tables.size(); i++) {
                            Map<String, Object> table = tables.get(i);
                            TableVO tableVO = new TableVO();
                            tableVO.setRowNumber(i + 1);
                            tableVO.setTableName(table.get("TABLE_NAME").toString());
                            tableVO.setTableComment(table.get("TABLE_COMMENT").toString());
                            tableVO.setTableCollation(table.get("TABLE_COLLATION").toString());
                            tableVOObservableList.add(tableVO);
                        }
                        return 0;
                    }

                    @Override
                    protected void succeeded() {
                        dbTableTableView.setItems(tableVOObservableList);
                        MsgUtil.closeNewLoading();
                        super.succeeded();
                    }
                };
            }
        }.start();
    }

    public void setMainStage(Stage mainStage) {
        this.mainStage = mainStage;
    }
}
