package ink.wgink.code.factory.utils;

import com.sun.javafx.robot.FXRobot;
import com.sun.javafx.robot.impl.FXRobotHelper;
import javafx.application.Platform;
import javafx.beans.property.ReadOnlyBooleanProperty;
import javafx.beans.property.ReadOnlyDoubleProperty;
import javafx.beans.property.ReadOnlyObjectProperty;
import javafx.beans.property.ReadOnlyStringProperty;
import javafx.concurrent.Service;
import javafx.concurrent.Task;
import javafx.concurrent.Worker;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.geometry.Rectangle2D;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.stage.Modality;
import javafx.stage.Screen;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

import javax.xml.soap.Text;
import java.io.IOException;
import java.util.*;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: MessageUtil
 * @Description: 消息工具
 * @Author: WangGeng
 * @Date: 2021/3/3 21:25
 * @Version: 1.0
 **/
public class MsgUtil {

    public final static List<Stage> LOADING_LIST = new ArrayList<>();

    /**
     * 错误
     *
     * @param msg
     */
    public static void errorAlert(String msg) {
        alert(AlertType.NONE, msg);
    }

    /**
     * 确定
     *
     * @param alertType
     * @param msg
     */
    public static void alert(Alert.AlertType alertType, String msg) {
        Alert alert = new Alert(alertType, msg, ButtonType.CLOSE);
        alert.setTitle("提示");
        alert.getDialogPane().setPrefWidth(150);
        alert.show();
    }

    /**
     * 等待
     *
     * @throws IOException
     */
    public static void loading(Stage parentStage) throws IOException {
        FXMLLoader fxmlLoader = new FXMLLoader(MsgUtil.class.getResource("/route/loading.fxml"));
        Stage stage = new Stage(StageStyle.TRANSPARENT);
        stage.setScene(new Scene(fxmlLoader.load()));
        stage.initModality(Modality.APPLICATION_MODAL);
        stage.setAlwaysOnTop(true);
        if (parentStage != null) {
            stage.setX(parentStage.getX() + (parentStage.getWidth() - 200) / 2);
            stage.setY(parentStage.getY() + (parentStage.getHeight() - 100) / 2);
        }
        stage.show();
        LOADING_LIST.add(0, stage);
    }

    /**
     * 关闭最新的Loading
     */
    public static void closeNewLoading() {
        if (LOADING_LIST.isEmpty()) {
            return;
        }
        LOADING_LIST.get(0).close();
        LOADING_LIST.remove(0);
    }

    /**
     * 关闭全部Loading
     */
    public static void closeAllLoading() {
        for (Stage stage : LOADING_LIST) {
            stage.close();
        }
        LOADING_LIST.clear();
    }

}
